/*
**  CXSC is a C++ library for eXtended Scientific Computing (V 2.2)
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2006 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  F. Blomquist, University of Wuppertal, 19.09.2007;
*/

/* CVS $Id: lx_interval.cpp,v 1.4 2006/12/06 18:36:17 cxsc Exp $ */

#include "lx_interval.hpp"
#include "lx_imath.hpp"
#include "idot.hpp"
#include <iostream>

namespace cxsc {

l_interval times2powr(const l_real &a, const real &r)
// b = times2powr(a,r); returns an inclusion of a*2^r <= b;  
// This inclusion holds only, if the following condition is
// fulfilled:
// ************************************************************** 
// *     For r in [-2100,+2100] r must be an integer value;     *
// **************************************************************
// The above condition is NOT tested in this function !!
// If r lies outside [-2100,+2100] then it holds a*2^r <= b for
// any real number, unless an overflow occurs.
// The function is for the internal use only!
// Blomquist, 25.02,2008;
{
    l_interval res(0),ai;
    double dbl;
    int ex;

    if ( a != 0 )
    {
	ai = l_interval(a);

	if (abs(r)<2147483647)
	{
	    dbl = _double(r);
	    ex = (int) dbl;
	    Times2pown(ai,ex);
	    res = ai;
	}
	else
	    if (r < -2147483645)
	    {
		    Times2pown(ai,-3000);
		    res = ai;
	    }
	    else
		   Times2pown(ai,3000); // produces overflow!
    }
    return res;
}

lx_interval::lx_interval(const real& n, const string &s) throw()
// Constructor:
// (10^n, string) --- > (ex,li) vom Typ lx_interval
// string s must be of the form:  
//        s = "[1.234567,1.234567]";   or:  s = "[0,0]";
// or:    s = "[1.234567e+20,1.234568e+20]";
// or:    s = "[1.234567e-20,1.234568e-20]";
// or:    s = "[0,1]";   or: s = "[-1,0]";  or: s = "[-1,+1]";
// (ex,li) is an optimal inclusion of  10^n*s;

// Not allowed strings:
//        s = "1.234567";     or:   s = "1.234567e+20";
// or:    s = "[1.2,1.2";     or:   s = "1.2,1.2]";
// or:    s = "+[1.2,1.2]";   or:   s = "-[1.2,1.2]";

{
	const real c1 = 3.3219280948873623; // c1 = ln(10)/ln(2)
	const real c2 = 2.711437152598000E+015;
	const real c3 = 10000.0;
	l_interval x(0);
	lx_interval u,v;
	real y,ay,r;
	int p,q,stagsave;
	bool neg;
	
	stagsave = stagprec;

	if (!Is_Integer(n))
		cxscthrow(REAL_NOT_ALLOWED(
					 "lx_interval(const real&, const string& )"));
   // Check for brackets [...] in string s:
	string str1 = "[", str2 = "]";
	q = s.find_first_of(str1);
	p = s.find_first_of(str2);
	if (q == -1 || p == -1)
		cxscthrow(NO_BRACKETS_IN_STRING(
					 "lx_interval(const real&, const string& )"));
	stagprec = stagprec +1;	
	x = x + 0; // To get StagPrec(x) = actual value of stagprec;
	s >> x;    // [a,b] <= x; x of type l_interval;

	if (x==0)
	{
		li = 0;  ex = 0;
	}
	else
	{
		neg = (n<0);
		y = n*c1;
		y = floor(_double(y));
		ay = abs(y);  r = abs(n);
		if (ay>=Max_Int_R)
			cxscthrow(REAL_NOT_ALLOWED(
					 "lx_interval(const real&, const string& )"));
		u = power( lx_interval(0,l_interval(10)),r );
		v = lx_interval(ay,l_interval(1));
		if (r > c2)
		{
			v = lx_interval(ay-c3,l_interval(1));
			times2pown(u,-c3);
		}
      if (neg) u = v/u;
		else u = u/v;
		u = u*lx_interval(0,x);
		li = li_part(u);
		r = expo(u);
		ex = add_real(y,r);
	}
	stagprec = stagsave;
	li = adjust(li);
}

void scale_down(lx_interval &a)
// scale_down skaliert a moeglichst weit nach unten, ohne
// dass dabei binaere Stellen verloren gehen.
// Der mathematische Wert von a bleibt dabei erhalten, d.h.
// 2^(a.ex)*a.li  hat vor und nach der Skalierung stets den
// gleichen mathematischen Wert.
{
    int d;
    d = -1021 - expo_sm(a.li);
    // In case of a.li=0 we get d>0, and so nothing is done! 
    if (d<0)
    {
	Times2pown(a.li,d); // a.li = a.li * 2^d
	a.ex = sub_real(a.ex,d);
    }
}

void scale_up(lx_interval &a)
// scale_up skaliert a.li moeglichst weit nach oben.  
// Nach der Skalierung wird jedoch  a.li + a.li  keinen Overflow 
// erzeugen.
// 2^(a.ex)*a.li  hat fuer d>0 vor und nach der Skalierung stets
// den gleichen Wert. Nur im Fall d<0 kann es durch die 
// Skalierung nach unten nur zu einer minimalen Aufblaehung kommen,
// denn im Fall  d<0  nimmt d nur die Werte d = -1,-2 an.   
{
    int d;
    d = 1022 - expo_gr(a.li);
    if (d>-4) // if (d>-3) is also possible!
    {
	    Times2pown(a.li,d);      // a.li = a.li * 2^d
	    a.ex = sub_real(a.ex,d); // to keep 2^(a.ex)*a.li constant.
    }
}

l_interval & l_interval::operator = (const lx_interval &a) throw()
{
    int exa;
    real p(expo(a)),exr;
    l_interval al(li_part(a));
	 l_real lrI(Inf(al)), lrS(Sup(al));  // Neu !!!!!
    exa = expo_gr(al);
    if (exa<-100000) return (*this) = al; // al = 0;
    exr = p + exa;
    if (exr > 1024)
	cxscthrow(OVERFLOW_ERROR("LX_INTERVAL_UNREALIZABLE_AS_L_INTERVAL: l_interval & operator = (const lx_interval &a)"));
    Times2pown(al,p);
    if (Inf(al)<0 && lrI>=0)
       al = SetInf(al,l_real(0));
	 if (Sup(al)>0 && lrS<=0)
       al = SetSup(al,l_real(0));
    return (*this) = al;
}

interval & interval::operator = (const lx_interval &a) throw()
{
    l_interval al;
    interval z;
    al = a;
    z = al;
    return (*this) = z;
}

lx_interval expo2zero(const lx_interval &a) throw(OVERFLOW_ERROR)
// The exponent to base 2 of the object a is set to zero.
// The return value res is an inclusion of the object a
// unless an overflow occurs. 
{
    lx_interval res(0);
    l_interval la(li_part(a));
    int exa(expo_gr(la));
    real na(expo(a)), r;

    if (exa > -100000) // a != 0;
	{
	    r = add_real(exa,na);
	    if (r > 1024)
		cxscthrow(OVERFLOW_ERROR(
			      "lx_interval expo2zero(lx_interval &)"));
	    Times2pown(la,na);
	    res = lx_interval(0.0,la);
	}

    return res;
}

lx_interval::lx_interval(const lx_real& a, const lx_real& b) throw()
{
    lx_real a_(a),b_(b);
    l_real lr;
    l_interval h;
    bool a_zero(eq_zero(a)), b_zero(eq_zero(b));
    real na,nb,d;
    if (a_zero || b_zero)
	if (a_zero) 
	{
	    ex = expo(b);
	    li = l_interval(0,lr_part(b));
	} else
	{
	    ex = expo(a);
	    li = l_interval(lr_part(a),0);
	}
    else
    {  // Now we have:  a,b != 0:
	scale_up(a_);
	scale_up(b_);
	na = expo(a_);  nb = expo(b_);
	if (na >= nb)
	{
	    if (na==nb) // the most common case!
	    {
		ex = na;
		li = l_interval(lr_part(a_),lr_part(b_));
	    }
	    else // na > nb:
	    {
		d = na - nb;
		if (d>Max_Int_R) 
		    d = Max_Int_R;
		ex = na;
		lr = lr_part(b_);
		h = times2powr(lr,-d);
		li = l_interval(lr_part(a_),Sup(h)); 
	    }
	}
	else // na < nb:
	{
	    d = nb - na;
	    if (d>Max_Int_R) 
		d = Max_Int_R;
	    ex = nb;
	    lr = lr_part(a_);
	    h = times2powr(lr,-d);
	    li = l_interval(Inf(h),lr_part(b_));
	}
    }
}

lx_interval operator + (const lx_interval &a, const lx_interval &b) throw()
{
    const real c1 = 10000.0;
    int stagsave = stagprec,
	stagmax = 39,
	exa,exb;
    real sa,sb,p,n;

    if (stagprec>stagmax) stagprec = stagmax;

    l_interval a1,b1;
    lx_interval res,A(a),B(b);
    interval z;

    A = Blow(A);  // If |a| is too small, a is inflated. 
    B = Blow(B);  // A <= Blow(A) is guaranteed!
    // With these inflations a+b <= A+B is guaranteed and
    // A+B is calculated without an error message unless
    // |A| or |B| are too great. 
    a1 = li_part(A);  b1 = li_part(B);
    exa = expo_gr(a1);
    exb = expo_gr(b1);

    if (exa<-100000) return b;  // summand a = 0
    if (exb<-100000) return a;  // summand b = 0

    // a,b <> 0:
    sa = add_real(exa,expo(A)); 
    sb = add_real(exb,expo(B));
    if (sa > sb) // |a| >= |b|
    {
	p = 1022 - exa; // p >= 0
	Times2pown(a1,p);
	n = sub_real(expo(A),p);
	if (n>0 && expo(B)<sub_real(n,c1))
	{
	    z = b1;
	    if (Sup(z)<=0.0)
		b1 = l_interval(-MinReal,0);
	    else
		if (Inf(z)>=0.0)
		b1 = l_interval(0,MinReal);
		else
		    b1 = l_interval(-MinReal,MinReal);
	}
	else
	{
	    p = sub_real(expo(B),n); 
	    Times2pown(b1,p);
	}
    } else
    {
	p = 1022 - exb; // p >= 0
	Times2pown(b1,p);
	n = sub_real(expo(B),p);
	if (n>0 && expo(A)<sub_real(n,c1))
	{
	    z = a1;
	    if (Sup(z)<=0.0)
		a1 = l_interval(-MinReal,0);
	    else
		if (Inf(z)>=0.0)
		a1 = l_interval(0,MinReal);
		else
		    a1 = l_interval(-MinReal,MinReal);
	}
	else
	{
	    p = sub_real(expo(A),n);
	    Times2pown(a1,p);
	}
    }
    a1 = a1 + b1; // normal staggered addition
    res = lx_interval(n,a1);

    stagprec = stagsave;
    res = adjust(res);
    return res;
}

lx_interval operator * (const lx_interval &a, const lx_interval &b) throw()
{
    int stagsave = stagprec,
	stagmax = 39,
	exa,exb,d,D,k;
    real na,nb,diff;

    if (stagprec > stagmax) stagprec = stagmax;

    l_interval al,bl;
    lx_interval a_(a),b_(b),res(0);

    a_ = Blow(a_); // a_ is inflated, if |a_| is too small
    b_ = Blow(b_); // a_ = [0,0] remains unchanged 

    scale_down(a_);  scale_down(b_);

    al = li_part(a_);  bl = li_part(b_);
    exa = expo_gr(al);
    exb = expo_gr(bl);

    if (exa>-100000 && exb>-100000) // Beide Faktoren sind <> 0:
    {
	if (exa+exb <= 1022)
	{
	    if (exa<0)
	    {
		Times2pown(al,-exa);  
		na = add_real(expo(a_),exa);  // a = 2^na * al;
		d = 1022 - exb;
	    }
	    else // exa >= 0
	    {
		na = expo(a_);
		d = 1022 - (exa+exb);
	    }
	    Times2pown(bl,d);
	    nb = sub_real(expo(b_),d);  // b = 2^nb * bl;
	}
	else // exa+exb > 1022
	{
	    d = exa + exb - 1022;  // d > 0;
	    if (exa>exb)
	    {
		D = exa - exb; // D > 0;
		if (d <= D)
		{
		    Times2pown(al,-d);
		    na = add_real(expo(a_),d); // a = 2^na * al;
		    nb = expo(b_);
		}
		else // d > D;
		{
		    k = d - D;
		    if (k%2 != 0) k++;
		    k = k/2;
		    Times2pown(bl,-k);
		    nb = add_real(expo(b_),k); // b = 2^nb * bl;
		    k = k + D;
		    Times2pown(al,-k);
		    na = add_real(expo(a_),k); // a = 2^na * al;
		}
	    }
	    else // exb >= exa
	    {
		D = exb - exa; // D > 0;
		if (d <= D)
		{
		    Times2pown(bl,-d);
		    nb = add_real(expo(b_),d); // b = 2^nb * bl;
		    na = expo(a_);
		}
		else // d > D
		{
		    k = d - D;
		    if (k%2 != 0) k++;
		    k = k/2;
		    Times2pown(al,-k);
		    na = add_real(expo(a_),k);  // a = 2^na * al;
		    k = k + D;
		    Times2pown(bl,-k);
		    nb = add_real(expo(b_),k);  // b = 2^nb * bl;
		}
	    }
	}
	al = al*bl;      // normal staggered multiplication

	if (na+nb<-Max_Int_R) // Underflow:
	{
	    diff = (Max_Int_R + na) + nb; // diff < 0;
	    Times2pown(al,diff);

	    if (Inf(al)>0)
		SetInf(al,0.0);
	    else
		if (Sup(al)<0)
		    SetSup(al,0.0);

	    res = lx_interval(Max_Int_N,al);
	}
	else // Without underflow
	{
	    na = add_real(na,nb); // Hier normale Rechnung!
	    res = lx_interval(na,al);
            // Caution: This else-block can deliver an interval res,
            // with a so small |res| value , that for example  
            // res * res produces an error message. To avoide this 
            // message, res will be sufficiently inflated:

	    if (na<Max_Int_N) // Only in extreme cases!
	    {
		if (Inf(al)>0)
		    SetInf(al,0.0);
		else
		    if (Sup(al)<0)
			SetSup(al,0.0);

		res = lx_interval(Max_Int_N,al);
	    }
	}

    }
    stagprec = stagsave;
    res = adjust(res);
    return res;
} // operator *

lx_interval operator / (const lx_interval &a, const lx_interval &b) 
                              throw(ERROR_LINTERVAL_DIV_BY_ZERO)
{
    int stagsave = stagprec,
	stagmax = 39,
	exa,exb,d;
    real na,nb,diff;
    if (stagprec>stagmax) stagprec = stagmax;

    interval z;
    l_interval al,bl;
    lx_interval a_(a),b_(b),res;

    a_ = Blow(a_); // a_ is inflated, if |a_| is too small
    b_ = Blow(b_); // a_ = [0,0] remains unchanged 

    scale_down(a_);  
    scale_down(b_);
    al = li_part(a_);  bl = li_part(b_);

    exa = expo_gr(al);
    exb = expo_gr(bl);

    if (exb < -100000) 
	cxscthrow(ERROR_LINTERVAL_DIV_BY_ZERO("lx_interval operator/(const lx_interval &a, const lx_interval &b)"));

    if (exa < -100000) return a;
    // Now it holds:  a,b <> 0:

    bool bl_point( point_intv(bl) );

    // First: scaling the nominator al to 1022:
    d = 1022 - exa;
    Times2pown(al,d);   // expo_gr(al) = 1022;
    na = sub_real(expo(a_),d);  // a = 2^na * al;

    if (bl_point) 
    {
	if (0<=exb && exb<=511) 
	    nb = expo(b_);
	else 
	    if (exb<0)
	    {  // scaling the denominator bl to 0:
		Times2pown(bl,-exb);
		nb = add_real(expo(b_),exb);  // b = 2^nb * bl;
	    }
	    else // exb > 511;
                 // Now scaling the denominator bl to 511:
	    {
		d = 511 - exb;
		Times2pown(bl,d);
		nb = sub_real(expo(b_),d);
	    }
    }
    else // scaling the denominator bl to 511:
    {
	d = 511 - exb;
	Times2pown(bl,d);
	nb = sub_real(expo(b_),d);
    }

    z = al;
    al = al / bl; // Normal staggered division; 
    // However, with too wide interval operands an overflow is possible:
    if ( (al==0 && z!=0) || Inf(al)>Sup(al) )
	cxscthrow(OVERFLOW_ERROR("TOO_WIDE_INTERVAL_OPERANDS_IN: lx_interval operator / (const lx_interval &a, const lx_interval &b)"));

    if (na-nb<-Max_Int_R) // Underflow:
    {
	diff = (Max_Int_R - nb) + na; // diff < 0;
	Times2pown(al,diff);

	if (Inf(al)>0)
	    SetInf(al,0.0);
	else
	    if (Sup(al)<0)
		SetSup(al,0.0);

	res = lx_interval(Max_Int_N,al);
        // +2100 delivers an additional inflation
        // to avoide too small results
    }
    else
    {   // Without underflow:
	na = sub_real(na,nb); // Hier normale Rechnung!
	res = lx_interval(na,al);
	// Caution: This else-block can deliver an interval res,
	// with a so small |res| value , that for example  
	// res * res produces an error message. To avoide this 
	// message, res will be sufficiently inflated:

	if (na<Max_Int_N) // Only in extreme cases!
	{
	    if (Inf(al)>0)
		SetInf(al,0.0);
	    else
		if (Sup(al)<0)
		    SetSup(al,0.0);

	    res = lx_interval(Max_Int_N,al);
	}
    }

    stagprec = stagsave;
    res = adjust(res);
    return res;
}

// ----------------------- Input --------------------------------------------

std::string & operator >> (std::string &s, lx_interval &a) throw()
// Writes string s to variable a of type lx_interval 
// and returns an empty string s;
// Example:  s = "{-4000,[2,2]}" delivers an interval a
// with:    10^(-4000)*[2,2] <= a;
// Whitespaces are NOT allowed in s ! 
{
    l_interval la;
    real exr;
    int i;

    s = skipwhitespacessinglechar (s, '{'); 
    s >> exr;
    s = skipwhitespacessinglechar (s, ','); 
    i = s.find("]");
    s.erase(i+1); 
    a = lx_interval(exr,s);
    s = "";

    return s;
}

void operator >> (const std::string &s, lx_interval &a) throw()
{
// Writes strings s to variable a of type lx_interval;
    std::string r(s);
    r >> a;
}

void operator >> (const char *s, lx_interval &a) throw()
{
    std::string r(s);
    r >> a;
}

bool StrContains(const string &s, const char &a, const char &b)
// returns 1, if s contains char a or char b, otherwise 0;
{
    int ia,ib;
    ia = s.find(a,0);
    ib = s.find(b,0);
    return (ia>=0 || ib>=0);
}

std::istream & operator >> (std::istream &s, lx_interval &a) throw()
// s must have the form: { p,[r1,r2] }, where r1<=r2 are real values
// and the exponent p of type real must be an integer value to base 10;
// The integer value condition and r1<=r2 are tested in the constructor
// call:   a = lx_interval(exr,str);
// Blomquist, 27.01.2008; 
{
    char c;
    real exr;
    string str,sh;

    skipeolnflag = inpdotflag = true;
    c = skipwhitespacessinglechar (s, '{');
    if (inpdotflag) s.putback(c);
    s >> SaveOpt >> exr; 
    c = skipwhitespacessinglechar (s, ',');
    if (inpdotflag) s.putback(c);
    skipeolnflag = inpdotflag = true;
    s >> str >> RestoreOpt;

    while ( !StrContains(str,']','}') )
    {
	c = skipwhitespaces (s);
	if (inpdotflag && c != '}') s.putback(c);
	if (c == '}') break;
	if (c == ' ') break;
	s >> sh;
	str = str + sh;
    }

    a = lx_interval(exr,str);

    if (!waseolnflag)
    {
	skipeolnflag = false; inpdotflag = true;
	c = skipwhitespaces (s);
	if (inpdotflag && c != '}')
	    s.putback(c);
    }

    return s;
}

// ----------------------- Output --------------------------------------------

void Bin2Dec(const lx_interval& a, real& p, l_interval& m)
// Calculates for a given interval a of type lx_interval the
// exponent p and the classic staggered interval m of type
// l_interval, so that the inclusion 
//        a <= 10^p * m        is guaranteed.
// Blomquist, 26.09.2008;
{
	const real c1 = 0.301029995663981195;
	const real c2 = 9.007199254738000E+015;
	const real c3 = 10000.0;
	l_interval x;
	lx_interval u,v;
	real y,k,l;
	int stagsave;
	bool neg;
	
	stagsave = stagprec;
	stagprec = stagprec + 1;
		
	x = li_part(a);
	y = expo(a);
	neg = y<0;
	if (x==0)
	{
		p = 0;  m = 0;
	}
	else
	{
		p = y*c1;
		p = floor(_double(p));
		y = abs(y);
		u = lx_interval(y,l_interval(1));
		v = power( lx_interval(0,l_interval(10)),abs(p) );
		if (y > c2)
		{
			u = lx_interval(y-c3,l_interval(1));
			times2pown(v,-c3);
		}
		if (neg) u = v/u;
		else u = u/v;
		u = u*lx_interval(0,x);
		m = li_part(u);
		k = expo(u);
		l = k*c1;
		l = floor(_double(l)) + 1;
		p = p+l;
		u = lx_interval(k,l_interval(1)) /
				power( lx_interval(0,l_interval(10)),l );
		u = u * lx_interval(0,m);
		k = expo(u);
		m = li_part(u);
		Times2pown(m,k);
	}
	stagprec = stagsave;
	m = adjust(m);
}  // Bin2Dec(...)

std::ostream& operator << (std::ostream& s,const lx_interval& a) throw()
// An interval a of type lx_interval is written to the 
// output channel in the decimal form: 
// { 10**(p), l_interval m } = 10^p * m;
// The following inclusion is realized:   a <= 10^p * m;
// Blomquist, 18.01.2008;
{
	real p;
	l_interval m;

	Bin2Dec(a,p,m);
	
   s << "{ " 
     << "10**("
     << SaveOpt << SetPrecision(0,0) << Fixed << p << RestoreOpt
     << ")" 
     << "*" 
     << m       
     << " }";
   return s;
}

std::string & operator << (std::string &s, const lx_interval &a) throw()
// The value of a variable a of type lx_interval is copied to a string s.
// s has the form:  {2**(ex), li} = 2^ex * li;
{  
    std::stringstream ss;
    string str;
    s+="{2**(";
    ss << SaveOpt << SetPrecision(0,0) << Fixed << a.ex << RestoreOpt;
    ss >> str;
    s += str;
    s+=")*";
    s << a.li; 
    s+='}';
    return s;
}


// ----- Help Functions, declared in lx_interval.hpp outside ---------------
// --------------------- the class lx_interval------------------------------

l_interval point_max(void)
// returns a staggered point interval with maximum exponent 1020,
// whereby nearly all mantissa bits are set. 
{
    l_real lr;
    l_interval li = sqrt(l_interval(3.995)), res(li);
    times2pown(li,1019);
    lr = Inf(li);
    lr = ((lr + Inf(res)) + MinReal) + minreal;
    res = l_interval(lr);
    return res;
}

l_interval point_any(int n)
// returns a staggered point interval with exponent n,
// whereby nearly all mantissa bits are set.
// -1074 <= n <= +1020; 
{
    int n_(n);
    l_interval res;

    if (n_>1020)  n_ = 1020;
    if (n_<-1074) n_ = -1074;
    res = point_max();
    times2pown(res,n_ - 1020);
    return l_interval(Inf(res+MinReal));
}

l_interval wide_max(void)
// returns a staggered interval a with maximum exponent 1020
// and diam(a)>0, whereby nearly all mantissa bits are set. 
{
    l_interval a = point_max();
    return a + interval(MinReal,2*MinReal);
}

l_interval wide_any(int n)
// returns a staggered point interval a with exponent n,
// and diam(a)>0, whereby nearly all mantissa bits are set.
// -1074 <= n <= +1020;
{
    l_interval a = point_any(n);
    return a + interval(MinReal,2*MinReal);
}

} // namespace cxsc
